﻿namespace Phoenix.Model.Bll.Domain
{
    using System;
    using System.ComponentModel;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;

    /// <summary>
    /// Represent a macro for the medical card editor. That's a predifined text to add into the text editor
    /// </summary>
    public class Macro : Poco
    {
        #region Fields

        private string bindedText = string.Empty;
        private string text = string.Empty;
        private string title = string.Empty;

        #endregion Fields

        #region Enumerations

        /// <summary>
        /// Provide predifined macros
        /// </summary>
        public enum Predifined
        {
            /// <summary>
            /// Gets the date of today
            /// </summary>
            [Description("Today's date")]
            Today,
            /// <summary>
            /// Get the time.
            /// </summary>
            [Description("Current time")]
            Now,
            /// <summary>
            /// Get the age of the patient
            /// </summary>
            [Description("Age of the patient")]
            PatientAge,
            /// <summary>
            /// Gets the name of the patient
            /// </summary>
            [Description("Name of the patient")]
            PatientName,
            /// <summary>
            /// Gets the date of birth of the patient
            /// </summary>
            [Description("Birthdate")]
            BirthDate,
        }

        #endregion Enumerations

        #region Properties

        /// <summary>
        /// Gets or sets macro's text after binding.
        /// That's with the calculated values.
        /// </summary>
        /// <value>The binded text.</value>        
        public virtual string BindedText
        {
            get { return this.bindedText.Trim(); }
            set { this.Set(this, "BindedText", ref this.bindedText, value); }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is configured.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is configured; otherwise, <c>false</c>.
        /// </value>
        public virtual bool IsConfigured
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets or sets the text to insert into the text editor.
        /// </summary>
        /// <value>The text.</value>
        public virtual string Text
        {
            get { return this.text; }
            set { this.Set(this, "text", ref this.text, value); }
        }

        /// <summary>
        /// Gets or sets the title.
        /// </summary>
        /// <value>The title.</value>
        public virtual string Title
        {
            get { return this.title; }
            set { this.Set(this, "title", ref this.title, value); }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Get value for predifined tags to be replaced by calculated values
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>The tag for the specified predifined macro</returns>
        public static string GetTag(Predifined value)
        {
            switch (value)
            {
                case Predifined.Today:
                    return "<Today>";
                case Predifined.Now:
                    return "<Now>";
                case Predifined.PatientAge:
                    return "<PatientAge>";
                case Predifined.PatientName:
                    return "<PatientName>";
                case Predifined.BirthDate:
                    return "<DateOfBirth>";
                default:
                    return Assert.FailsOnEnumeration(value) as string;
            }
        }

        /// <summary>
        /// Bindthe specified patient to the current macro.
        /// That's put the name of the patient or his age into the macro.
        /// </summary>
        /// <param name="patient">The patient.</param>
        public virtual void Bind(Patient patient)
        {
            string now = string.Format("{0:00}:{1:00}:{2:00}", DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
            Assert.IsNotNull(patient, "patient", "Impossible to configure the macro if no patient is linked to this macro!");
            this.IsConfigured = true;
            this.BindedText = this.Text;
            this.BindedText = this.BindedText.Replace(Macro.GetTag(Predifined.Today), DateTime.Today.ToShortDateString());
            this.BindedText = this.BindedText.Replace(Macro.GetTag(Predifined.Now), now);
            this.BindedText = this.BindedText.Replace(Macro.GetTag(Predifined.PatientName), patient.ToString());
            this.BindedText = this.BindedText.Replace(Macro.GetTag(Predifined.PatientAge), patient.DateOfBirth.GetAge().ToString());
            this.BindedText = this.BindedText.Replace(Macro.GetTag(Predifined.BirthDate), patient.DateOfBirth.ToShortDateString());
        }

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return string.Format("Macro: '{0}'"
                , this.Title);
        }

        #endregion Methods
    }
}