﻿namespace Phoenix.Model.Management
{
    using Phoenix.Contracts;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;
    using Phoenix.Model.Dal;
    using Phoenix.Presenter.Interfaces.Input;

    /// <summary>
    /// Provides features to manage the keywords.
    /// </summary>
    public class KeywordModel : Manager<KeywordGroup>
    {
        #region Fields

        private IKeywordInputPresenter presenter;
        private IRepository<KeywordGroup> repository;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="KeywordModel"/> class.
        /// </summary>
        /// <param name="presenter">The view.</param>
        /// <param name="repository">The repository.</param>
        public KeywordModel(IKeywordInputPresenter presenter, IRepository<KeywordGroup> repository)
            : base(repository)
        {
            this.presenter = presenter;
            this.presenter.Inserted += view_Added;
            this.presenter.Deleted += view_Deleted;
            this.presenter.Updated += view_Updated;
            this.repository = repository;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="KeywordModel"/> class.
        /// </summary>
        /// <param name="presenter">The view.</param>
        public KeywordModel(IKeywordInputPresenter presenter)
            : this(presenter, RepositoryFactory.Get<IKeywordGroupRepository>())
        {
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Loads the presenter and all the data.
        /// </summary>
        public void RefreshData()
        {
            this.presenter.KeywordGroups = this.Repository.GetAllRows();
        }

        /// <summary>
        /// Validates the data.
        /// </summary>
        /// <returns><c>True</c> if the data have passed the validation;
        /// otherwise <c>False</c></returns>
        public override bool ValidateData()
        {
            Assert.IsNotNull(this.presenter.ItemToManage, "ItemToManage");
            this.presenter.ClearError();
            bool result = true;
            if (string.IsNullOrEmpty(this.presenter.ItemToManage.Title))
            {
                this.presenter.Invalidate(KeywordGroupFailedOn.NoTitle, "The keyword group should have a title!");
                result = false;
            }
            if (this.presenter.ItemToManage.Keywords.Count <= 0)
            {
                this.presenter.Invalidate(KeywordGroupFailedOn.NoKeywords, "The group should have at least one keyword!");
                result = false;
            }
            return result;
        }

        private void view_Added(object sender, System.EventArgs e)
        {
            this.Repository.Create(this.presenter.ItemToManage);
            this.RefreshData();
        }

        private void view_Deleted(object sender, System.EventArgs e)
        {
            this.Repository.Delete(this.presenter.ItemToManage);
            this.RefreshData();
        }

        /// <summary>
        /// Handles the Updated event of the view control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void view_Updated(object sender, System.EventArgs e)
        {
            this.Repository.Update(this.presenter.ItemToManage);
            this.RefreshData();
        }

        #endregion Methods
    }
}