﻿namespace Phoenix.Presenter
{
    using System;
    using System.Collections.Generic;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Presenter.Interfaces;

    /// <summary>
    /// Application context of Phoenix
    /// </summary>
    public static class AppContext
    {
        #region Fields

        private static Dictionary<string, object> context = new Dictionary<string, object>();
        private static object sender = new object();

        #endregion Fields

        #region Constructors

        static AppContext()
        {
            Context.Add("PluginRepository", null);
            Context.Add("SessionPlugin", null);
            Context.Add("DefaultUser", new User());
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when some pieces of data are updated.
        /// </summary>
        public static event EventHandler DataChanged;

        /// <summary>
        /// Occurs when a global message is sent.
        /// </summary>
        public static event EventHandler<EventArgs<string>> GlobalMessageSent;

        /// <summary>
        /// Occurs when meetings are updated.
        /// </summary>
        public static event EventHandler MeetingsUpdated;

        /// <summary>
        /// Occurs when the user ask to load a session.
        /// </summary>
        public static event EventHandler<EventArgs<Patient>> SessionLoading;

        /// <summary>
        /// Occurs when a session is updated.
        /// </summary>
        public static event EventHandler<EventArgs<Patient>> SessionUpdated;

        /// <summary>
        /// Occurs when a week is selected in the meeting manager.
        /// </summary>
        public static event EventHandler<EventArgs<DateTime>> WeekSelected;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets a generic context.
        /// </summary>
        /// <value>The context.</value>
        public static Dictionary<string, object> Context
        {
            get { return context; }
        }

        public static User CurrentUser
        {
            get
            {
                return Context["DefaultUser"] as User;
            }
            set
            {
                Context["DefaultUser"] = value;
            }
        }

        public static string PluginRepositoryPath
        {
            get
            {
                var path = Context["PluginRepository"] as string;
                if (path == null) throw new ArgumentException("Please configure the directory of the plugins.");
                return path;
            }
            set
            {
                Context["PluginRepository"] = value;
            }
        }

        public static IManager[] SessionPlugins
        {
            get
            {
                return Context["SessionPlugin"] as IManager[];
            }
            set
            {
                Context["SessionPlugin"] = value;
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Clears the global message.
        /// </summary>
        public static void ClearGlobalMessage()
        {
            SendGlobalMessage("Ready.");
        }

        /// <summary>
        /// Called when some pieces of data are updated].
        /// </summary>
        public static void NotifyDataChanged()
        {
            DataChanged.Raise(sender);
        }

        /// <summary>
        /// Called when meeting are updated.
        /// </summary>
        public static void NotifyMeetingUpdated()
        {
            Assert.IsNotNull(sender, "sender");
            MeetingsUpdated.Raise(sender, EventArgs.Empty);
        }

        /// <summary>
        /// Called when user ask to load a session.
        /// </summary>
        /// <param name="patient">The patient.</param>
        public static void NotifySessionLoading(Patient patient)
        {
            Assert.IsNotNull(sender, "sender");
            SessionLoading.Raise(sender, new EventArgs<Patient>(patient));
        }

        /// <summary>
        /// Called when a session is updated.
        /// </summary>
        /// <param name="patient">The patient.</param>
        public static void NotifySessionUpdated(Patient patient)
        {
            Assert.IsNotNull(sender, "sender");
            SessionUpdated.Raise(sender, new EventArgs<Patient>(patient));
        }

        /// <summary>
        /// Called when a week is selected.
        /// </summary>
        /// <param name="from">Start of the selection.</param>
        public static void NotifyWeekSelected(DateTime from)
        {
            Assert.IsNotNull(sender, "sender");
            WeekSelected.Raise(sender, new EventArgs<DateTime>(from));
        }

        /// <summary>
        /// Sends message to all Global message listener.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="args">The args to format.</param>
        public static void SendGlobalMessage(object message, params object[] args)
        {
            Assert.IsNotNull(sender, "sender");
            var msg = string.Format(message.ToString(), args);
            GlobalMessageSent.Raise(sender, new EventArgs<string>(msg));
        }

        #endregion Methods
    }
}