﻿namespace Phoenix.Presenter.Sessions
{
    using System;
    using System.Collections.Generic;

    using log4net;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Interfaces.Manager;

    using StructureMap;

    /// <summary>  
    /// When a sessions is loaded and linked to a Patient it'll give on demand
    /// all the services link to the specified Patient.
    /// </summary>
    public sealed class Session : IEquatable<Session>
    {
        #region Fields

        private ILog Logger = LogManager.GetLogger(typeof(Session));

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Session"/> class.
        /// </summary>
        /// <param name="patient">The patient.</param>
        public Session(Patient patient)
        {
            Assert.IsNotNull(patient, "The patient shouldn't be null in the session.");
            this.Patient = patient;
            this.Managers = new List<IManager>();

            #region IPatientManager
            IPatientManagerPresenter pm = ObjectFactory.GetInstance<IPatientManagerPresenter>();
            pm.Patient = patient;
            this.NotifyPatientLoaded(pm);
            this.Managers.Add(pm);
            #endregion

            #region IBmiManager
            IBmiManagerPresenter bmiManager = ObjectFactory.GetInstance<IBmiManagerPresenter>();
            bmiManager.Patient = patient;
            this.Managers.Add(bmiManager);
            #endregion

            #region IFamilyManager
            //TODO refactor this...
            IFamilyManagerPresenter familyManager = ObjectFactory.GetInstance<IFamilyManagerPresenter>();
            familyManager.PatientLoading += (sender, e) => this.PatientSelected.Raise(this, e);
            familyManager.RelationDeleted += (sender, e) => this.Modified.Raise(this);
            familyManager.RelationInserted += (sender, e) => this.Modified.Raise(this);
            familyManager.RelationUpdated += (sender, e) => this.Modified.Raise(this);
            familyManager.Patient = patient;
            this.Managers.Add(familyManager);
            #endregion

            #region IMedicalCardManager
            IMedicalCardManagerPresenter medicalCardMan = ObjectFactory.GetInstance<IMedicalCardManagerPresenter>();
            medicalCardMan.Patient = patient;
            this.Managers.Add(medicalCardMan);
            #endregion

            #region IPictureManager
            IPictureManagerPresenter pictureManager = ObjectFactory.GetInstance<IPictureManagerPresenter>();
            pictureManager.Patient = patient;
            pictureManager.Usage = Usage.Insert;
            this.Managers.Add(pictureManager);
            #endregion

            #region IPrescriptionManager
            IPrescriptionManagerPresenter prescriptionManagerBis = ObjectFactory.GetInstance<IPrescriptionManagerPresenter>();
            prescriptionManagerBis.Patient = patient;
            this.Managers.Add(prescriptionManagerBis);
            #endregion

            #region IPathologyHistory
            IPathologiesHistoryPresenter pathologiesHistory = ObjectFactory.GetInstance<IPathologiesHistoryPresenter>();
            pathologiesHistory.Patient = patient;
            this.Managers.Add(pathologiesHistory);
            #endregion

            #region Plugins
            foreach (var plugin in AppContext.SessionPlugins)
            {
                plugin.Patient = patient;
                this.Managers.Add(plugin);
            }
            #endregion

            //Register to data update
            AppContext.DataChanged += (sender, e) => this.RefreshData();
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when the session was modified.
        /// </summary>
        public event EventHandler Modified;

        /// <summary>
        /// Occurs a patient is selected from the family manager.
        /// Typically to load the session of a parent.
        /// </summary>
        public event EventHandler<EventArgs<Patient>> PatientSelected;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets a value indicating whether this instance is closed.
        /// </summary>
        /// <value><c>true</c> if this instance is closed; otherwise, <c>false</c>.</value>
        public bool IsClosed
        {
            get;
            private set;
        }

        public List<IManager> Managers
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets or sets the patient of the sessions.
        /// </summary>
        /// <value>The patient.</value>
        public Patient Patient
        {
            get;
            private set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Closes the session and dispose all the GUI item related
        /// to the current session.
        /// </summary>
        public void Close()
        {
            this.Logger.Debug(string.Format("Closing session {0}", this.Patient.ID));
            this.IsClosed = true;

            foreach (var manager in this.Managers) manager.ShutDown();
        }

        /// <summary>
        /// Indicates whether the current object is equal to another object of the same type.
        /// </summary>
        /// <param name="other">An object to compare with this object.</param>
        /// <returns>
        /// true if the current object is equal to the <paramref name="other"/> parameter; otherwise, false.
        /// </returns>
        public bool Equals(Session other)
        {
            return (this.Patient.ID == other.Patient.ID);
        }

        /// <summary>
        /// Refreshes the current session.
        /// </summary>
        public void RefreshData()
        {
            foreach (var manager in this.Managers) manager.RefreshData();
        }

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return "Session of: " + this.Patient.ToString();
        }

        private void NotifyPatientLoaded(IPatientManagerPresenter pm)
        {
            try
            {
                pm.OnPatientLoaded();
            }
            catch (Exception ex)
            {
                //I swallow because it isn't worth notifying this on error
                this.Logger.Warn("Failed to notify the patient was loaded", ex);
            }
        }

        #endregion Methods
    }
}