﻿namespace Phoenix.View.Configuration
{
    using System;

    using Phoenix.Presenter.Search;
    using Phoenix.View.Properties;

    /// <summary>
    /// Contains the configuration for the SearchBox
    /// </summary>
    public class AppearanceConfiguration : IConfigurationMemento
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="AppearanceConfiguration"/> class.
        /// </summary>
        public AppearanceConfiguration()
        {
            this.SaveState();
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the heigth of the criterion panel for the doctorsearchbox.
        /// </summary>
        /// <value>The splitter distance.</value>
        public int DoctorCriterionPanelHeight
        {
            get
            {
                return Settings.Default.SearchBox_DoctorCriterionPanelHeight;
            }
            set
            {
                Settings.Default.SearchBox_DoctorCriterionPanelHeight = value;
            }
        }

        /// <summary>
        /// The user cant make search earlier than this date on the search box.
        /// </summary>
        /// <value>The minimum date.</value>
        public DateTime MinDate
        {
            get
            {
                return Settings.Default.General_MinimumDate;
            }
            set
            {
                Settings.Default.General_MinimumDate = value;
            }
        }

        /// <summary>
        /// Gets or sets the heigth of the criterion panel for the patient searchbox.
        /// </summary>
        /// <value>The splitter distance.</value>
        public int PatientCriterionPanelHeight
        {
            get
            {
                return Settings.Default.SearchBox_PatientCriterionPanelHeight;
            }
            set
            {
                Settings.Default.SearchBox_PatientCriterionPanelHeight = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indacating the picturesize threshold before warn
        /// on big size.
        /// </summary>
        /// <value>The picture size to warn.</value>
        public int PictureSizeToWarn
        {
            get
            {
                return Settings.Default.General_PictureSizeToWarn;
            }
            set
            {
                Settings.Default.General_PictureSizeToWarn = value;
            }
        }

        /// <summary>
        /// Gets or sets the search box default view.
        /// </summary>
        /// <value>The search box view.</value>        
        public SearchBox.SearchBoxView SearchBoxView
        {
            get
            {
                return (SearchBox.SearchBoxView)Enum.Parse(typeof(SearchBox.SearchBoxView), Settings.Default.SearchBox_DefaultView);
            }
            set
            {
                Settings.Default.SearchBox_DefaultView = value.ToString();
            }
        }

        /// <summary>
        /// Gets the state of this instance.
        /// </summary>
        /// <value>The state.</value>
        private Memento State
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Commits the changes done in the config file.
        /// </summary>
        public void CommitChanges()
        {
            Settings.Default.Save();
        }

        /// <summary>
        /// Rollbacks this instance to the previous saved state.
        /// </summary>
        public void Rollback()
        {
            this.SearchBoxView = this.State.SearchBoxView;
            this.PatientCriterionPanelHeight = this.State.SplitterDistance;
            this.MinDate = this.State.MinimumDate;
        }

        /// <summary>
        /// Saves the state of this instance.
        /// </summary>
        public void SaveState()
        {
            Memento state = new Memento();
            state.SearchBoxView = (SearchBox.SearchBoxView)Enum.Parse(typeof(SearchBox.SearchBoxView), Settings.Default.SearchBox_DefaultView);
            state.SplitterDistance = Settings.Default.SearchBox_PatientCriterionPanelHeight;
            state.MinimumDate = Settings.Default.General_MinimumDate;
            this.State = state;
        }

        #endregion Methods

        #region Nested Types

        /// <summary>
        /// Memento to save the AppearanceConfiguration state to a given moment.
        /// </summary>
        internal class Memento
        {
            #region Constructors

            /// <summary>
            /// Initializes a new instance of the <see cref="Memento"/> class.
            /// </summary>
            public Memento()
            {
            }

            #endregion Constructors

            #region Properties

            /// <summary>
            /// Gets or sets the minimum date.
            /// </summary>
            /// <value>The minimum date.</value>
            public DateTime MinimumDate
            {
                get;
                set;
            }

            /// <summary>
            /// Gets or sets the search box view.
            /// </summary>
            /// <value>The search box view.</value>
            public SearchBox.SearchBoxView SearchBoxView
            {
                get;
                set;
            }

            /// <summary>
            /// Gets or sets the state of the splitter.
            /// </summary>
            /// <value>The state of the splitter.</value>
            public int SplitterDistance
            {
                get;
                set;
            }

            #endregion Properties
        }

        #endregion Nested Types
    }
}