﻿namespace Phoenix.View.Helpers
{
    using System.Windows.Forms;

    using Phoenix.Model.Helpers;

    /// <summary>
    /// Provides features to test data validity in UserControls.
    /// </summary>   
    public static class GuiValidator
    {
        #region Methods

        /// <summary>
        /// Checks is an item is selected in the combo box.
        /// If no, execute the specified ErrorProvider with an
        /// explanation on the error.
        /// </summary>
        /// <param name="ep">The error provider.</param>
        /// <param name="cb">The combo box.</param>
        /// <returns>
        /// 	<c>True</c> if the value is valid. Otherwise returns <c>False</c>.
        /// </returns>
        public static bool CheckComboBox(ErrorProvider ep, ComboBox cb)
        {
            string msg;
            bool isValid = Check.DataSelector(cb.SelectedItem, out msg);

            if (!isValid) ep.SetError(cb, msg);
            return isValid;
        }

        /// <summary>
        /// Checks is the date of the specified DateTimePicker is valid.
        /// If not valid, execute the specified ErrorProvider with an
        /// explanation on the error.
        /// </summary>
        /// <param name="ep">The error provider.</param>
        /// <param name="dtp">The DateTimePicker to check.</param>        
        /// <returns><c>True</c> if the value is valid. Otherwise returns <c>False</c>.</returns>
        public static bool CheckDateTimePickerValue(ErrorProvider ep, DateTimePicker dtp)
        {
            string msg;
            bool isValid = Check.DateTimeIsInPast(dtp.Value, out msg);

            if (!isValid) ep.SetError(dtp, msg);
            return isValid;
        }

        /// <summary>
        /// Checks is the text of the specified TextBox is a valid email.
        /// If not valid, execute the specified ErrorProvider with an
        /// explanation on the error.
        /// </summary>
        /// <param name="ep">The error provider.</param>
        /// <param name="tb">The text box to check.</param>        
        /// <returns><c>True</c> if the value is valid. Otherwise returns <c>False</c>.</returns>
        public static bool CheckTextBoxEmail(ErrorProvider ep, TextBox tb)
        {
            string msg;
            bool isValid = Check.Email(tb.Text, out msg);

            if (!isValid) ep.SetError(tb, msg);
            return isValid;
        }

        /// <summary>
        /// Checks is the text of the specified TextBox is not a number.
        /// If not valid, execute the specified ErrorProvider with an
        /// explanation on the error.
        /// </summary>
        /// <param name="ep">The error provider.</param>
        /// <param name="tb">The text box to check.</param>        
        /// <returns><c>True</c> if the value is valid. Otherwise returns <c>False</c>.</returns>
        public static bool CheckTextBoxNotNumber(ErrorProvider ep, TextBox tb)
        {
            string msg;

            bool isValid = Check.TextIsNotNumeric(tb.Text, out msg);
            if (!isValid) ep.SetError(tb, msg);
            return isValid;
        }

        /// <summary>
        /// Checks is the text of the specified TextBox is valid.
        /// If not valid, execute the specified ErrorProvider with an
        /// explanation on the error.
        /// </summary>
        /// <param name="ep">The error provider.</param>
        /// <param name="tb">The text box to check.</param>        
        /// <returns><c>True</c> if the value is valid. Otherwise returns <c>False</c>.</returns>
        public static bool CheckTextBoxValueNotEmpty(ErrorProvider ep, TextBox tb)
        {
            string msg;
            bool isValid = Check.TextIsNotEmpty(tb.Text, out msg);
            if (!isValid) ep.SetError(tb, msg);
            return isValid;
        }

        #endregion Methods
    }
}