﻿namespace Phoenix.View.Helpers
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Linq;

    using MigraDoc.DocumentObjectModel;
    using MigraDoc.DocumentObjectModel.Tables;
    using MigraDoc.Rendering;

    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Presenter;
    using Phoenix.Translation;
    using Phoenix.View.Configuration;
    using Phoenix.View.Properties;
    using log4net;

    /// <summary>
    /// Create a pdf file with the history of the next meeting of a patient
    /// </summary>
    public class NextMeetingsPdf : LogObject
    {
        #region Fields

        private IniFile nini;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="NextMeetingsPdf"/> class.
        /// </summary>
        public NextMeetingsPdf()
        {
            this.Logger = LogManager.GetLogger(typeof(NextMeetingsPdf));
            this.PageSetup = new PageSetup();
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the file path.
        /// </summary>
        /// <value>The file path.</value>
        private string FilePath
        {
            get
            {
                return Settings.Default.MeetingHistoryPdf;
            }
        }

        private List<WorkMeeting> Meetings
        {
            get;
            set;
        }

        private PageSetup PageSetup
        {
            get;
            set;
        }

        private Patient Patient
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Gets the history for the specified patient.
        /// </summary>
        public void BuildPdf(Patient patient)
        {
            this.SetupPage();
            this.Patient = patient;
            this.Meetings = (from meeting in this.Patient.Meetings
                             where meeting.StartDate > DateTime.Today.AddDays(1)
                             select meeting).ToList();

            var document = this.Create();

            PdfDocumentRenderer renderer = new PdfDocumentRenderer(true, PdfSharp.Pdf.PdfFontEmbedding.Always);
            renderer.Document = document;
            renderer.RenderDocument();

            renderer.Save(this.FilePath);
            try
            {
                Process.Start(this.FilePath);
            }
            catch (Exception ex)
            {
                MessageBoxes.Warning("Impossible to display the pdf: " + ex.Message);
                this.Logger.Warn("Impossible to display the pdf", ex);
            }
        }

        /// <summary>
        /// Defines the styles used in the document.
        /// </summary>
        public void DefineStyles(Document document)
        {
            // Get the predefined style Normal.
            Style style = document.Styles["Normal"];
            // Because all styles are derived from Normal, the next line changes the
            // font of the whole document. Or, more exactly, it changes the font of
            // all styles and paragraphs that do not redefine the font.
            style.Font.Name = "Times New Roman";

            // Heading1 to Heading9 are predefined styles with an outline level. An outline level
            // other than OutlineLevel.BodyText automatically creates the outline (or bookmarks)
            // in PDF.

            style = document.Styles["Heading1"];
            style.Font.Name = "Tahoma";
            style.Font.Size = 14;
            style.Font.Bold = true;
            style.Font.Color = Colors.DarkBlue;
            style.ParagraphFormat.PageBreakBefore = false;
            style.ParagraphFormat.SpaceAfter = 6;

            style = document.Styles["Heading2"];
            style.Font.Size = 12;
            style.Font.Bold = true;
            style.ParagraphFormat.PageBreakBefore = false;
            style.ParagraphFormat.SpaceBefore = 6;
            style.ParagraphFormat.SpaceAfter = 6;

            style = document.Styles["Heading3"];
            style.Font.Size = 10;
            style.Font.Bold = true;
            style.Font.Italic = true;
            style.ParagraphFormat.SpaceBefore = 6;
            style.ParagraphFormat.SpaceAfter = 3;

            style = document.Styles[StyleNames.Header];
            style.ParagraphFormat.AddTabStop("16cm", TabAlignment.Right);

            style = document.Styles[StyleNames.Footer];
            style.ParagraphFormat.AddTabStop("8cm", TabAlignment.Center);

            // Create a new style called TextBox based on style Normal
            style = document.Styles.AddStyle("TextBox", "Normal");
            style.ParagraphFormat.Alignment = ParagraphAlignment.Justify;
            style.ParagraphFormat.Borders.Width = 2.5;
            style.ParagraphFormat.Borders.Distance = "3pt";
            style.ParagraphFormat.Shading.Color = Colors.SkyBlue;

            // Create a new style called TOC based on style Normal
            style = document.Styles.AddStyle("TOC", "Normal");
            style.ParagraphFormat.AddTabStop("16cm", TabAlignment.Right, TabLeader.Dots);
            style.ParagraphFormat.Font.Color = Colors.Blue;
        }

        private Document Create()
        {
            Document result = new Document();

            result.Info.Title
                = result.Info.Subject
                = string.Format("Meetings for {0}", this.Patient.FullName);

            result.Info.Author = AppContext.CurrentUser.ToString();

            this.DefineStyles(result);
            this.Fill(result);

            return result;
        }

        private void Fill(Document document)
        {
            this.SetupPage(document);
            document.AddSection();
            Paragraph p = document.LastSection.AddParagraph();
            p.Format.Alignment = ParagraphAlignment.Right;

            Address a = AppContext.CurrentUser.MainPractice.Address;
            p.AddText(string.Format("{1}{0}{2}{0}{3}{0}{4}"
            , Environment.NewLine
            , AppContext.CurrentUser.ToString()
            , AppContext.CurrentUser.Header
            , a.Street + " " + a.StreetNumber
            , a.PostalCode + " " + a.City));

            p = document.LastSection.AddParagraph("Meetings for {0}".Translate(this.Patient.FullName), "Heading1");
            this.FillMeetings(document);
        }

        private void FillMeetings(Document document)
        {
            Table table = new Table();
            table.Borders.Width = 0.75;

            Column column = table.AddColumn(Unit.FromCentimeter(3));
            column.Format.Alignment = ParagraphAlignment.Center;
            table.AddColumn(Unit.FromCentimeter(6));
            table.AddColumn(Unit.FromCentimeter(6));

            Row row = table.AddRow();
            row.Shading.Color = Colors.LightGray;
            row.Cells[0].AddParagraph("From".Translate());
            row.Cells[1].AddParagraph("To".Translate());
            row.Cells[2].AddParagraph("Remarks".Translate());

            foreach (var meeting in this.Meetings)
            {
                row = table.AddRow();
                row.Cells[0].AddParagraph(meeting.StartDate.ToString("dd/MM/yyyy HH:mm"));
                row.Cells[1].AddParagraph(meeting.EndDate.ToString("dd/MM/yyyy HH:mm"));
                row.Cells[2].AddParagraph(meeting.Remark);
            }

            table.SetEdge(0, 0, 3, this.Meetings.Count - 1, Edge.Box, BorderStyle.Single, 1.5, Colors.Black);
            document.LastSection.Add(table);
        }

        private Unit Get(string key)
        {
            var value = double.Parse(nini.Get(key));
            return new Unit(value, UnitType.Millimeter);
        }

        private void SetupPage()
        {
            try
            {
                nini = new IniFile("History_PageSetup");
                this.PageSetup.PageHeight = this.Get("PageHeight");
                this.PageSetup.PageWidth = this.Get("PageWidth");
                this.PageSetup.LeftMargin = this.Get("LeftMargin");
                this.PageSetup.RightMargin = this.Get("RightMargin");
                this.PageSetup.BottomMargin = this.Get("BottomMargin");
                this.PageSetup.TopMargin = this.Get("TopMargin");
            }
            catch (Exception ex)
            {
                throw new Exception("Impossible to setup the page to create the meetin history: "
                    + ex.Message
                    , ex); ;
            }
        }

        private void SetupPage(Document document)
        {
            document.DefaultPageSetup.PageHeight = this.PageSetup.PageHeight;
            document.DefaultPageSetup.PageWidth = this.PageSetup.PageWidth;
            document.DefaultPageSetup.LeftMargin = this.PageSetup.LeftMargin;
            document.DefaultPageSetup.RightMargin = this.PageSetup.RightMargin;
            document.DefaultPageSetup.BottomMargin = this.PageSetup.BottomMargin;
            document.DefaultPageSetup.TopMargin = this.PageSetup.TopMargin;
        }

        #endregion Methods
    }
}