﻿namespace Phoenix.View.Input
{
    using System;
    using System.Collections.Generic;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.View.Helpers;

    /// <summary>
    /// Manage <see cref="Pathology"/>
    /// </summary>
    public class PathologyInput : BasicInputWithComboBox, IPathologyInputPresenter
    {
        #region Fields

        private const string COMBOBOX_MEMBER = "Type";
        private const string VALUE1 = "Title";
        private const string VALUE2 = "Remark";

        private PathologyModel model;
        private Pathology pathology = new Pathology();
        private IList<PathologyType> types = new List<PathologyType>();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PathologyInput"/> class.
        /// </summary>
        public PathologyInput()
            : base(new Pathology(), VALUE1, VALUE2)
        {
            this.model = new PathologyModel(this);
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when user asked to delete an Drug
        /// </summary>
        public event EventHandler Deleted;

        /// <summary>
        /// Occurs when user asked to insert an Drug
        /// </summary>
        public event EventHandler Inserted;

        /// <summary>
        /// Occurs when user asked to update an Drug
        /// </summary>
        public event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public Pathology ItemToManage
        {
            get
            {
                this.pathology = this.Item as Pathology;
                this.pathology.Type = this.SelectedComboBoxItem as PathologyType;
                return pathology;
            }
            set
            {
                this.pathology = value;
                this.OnRefreshed();
                this.Bind(this.pathology, VALUE1, VALUE2, COMBOBOX_MEMBER);
            }
        }

        /// <summary>
        /// Gets or sets the types of pathology.
        /// </summary>
        /// <value>The types.</value>
        public IList<PathologyType> Types
        {
            get
            {
                return this.types;

            }
            set
            {
                this.types = value;
                this.SetDataSource(value, "Title");
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Sets an error message on validation failed.
        /// </summary>
        /// <param name="fail">Indicates where the validation failed.</param>
        /// <param name="message">The message to display.</param>
        public void SetError(PathologyFailedOn fail, string message)
        {
            switch (fail)
            {
                case PathologyFailedOn.NoName:
                    this.SetError(On.Value1, message);
                    break;
                case PathologyFailedOn.NoType:
                    this.SetError(On.Value3, message);
                    break;
                case PathologyFailedOn.Referenced:
                    PredifinedMsgBoxes.WarnOnReferencedItem();
                    break;
                default:
                    Assert.FailsOnEnumeration(fail);
                    break;
            }
        }

        /// <summary>
        /// Deletes the specified item to manage.
        /// </summary>
        /// <param name="itemToManage">The item to manage.</param>
        protected override void Delete(object itemToManage)
        {
            this.Deleted.Raise(this);
            this.OnRefreshed();
            AppContext.NotifyDataChanged();
        }

        /// <summary>
        /// Inserts the specified item to manage.
        /// </summary>
        /// <param name="itemToManage">The item to manage.</param>
        protected override void Insert(object itemToManage)
        {
            this.Inserted.Raise(this);
            this.OnRefreshed();
            AppContext.NotifyDataChanged();
        }

        /// <summary>
        /// Updates the specified item to manage.
        /// </summary>
        /// <param name="itemToManage">The item to manage.</param>
        protected override void Update(object itemToManage)
        {
            this.Updated.Raise(this);
            this.OnRefreshed();
            AppContext.NotifyDataChanged();
        }

        /// <summary>
        /// Validates the data.
        /// </summary>
        /// <returns>
        /// Returns <c>true</c> if the data is valid; otherwise <b>false</b>
        /// </returns>
        protected override bool ValidateData()
        {
            return this.model.ValidateData();
        }

        #endregion Methods
    }
}