﻿namespace Phoenix.View.Input
{
    using System;
    using System.Drawing;
    using System.Linq;
    using System.Windows.Forms;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Translation;
    using Phoenix.View.Helpers;
    using Phoenix.View.Helpers.Controls;

    /// <summary>
    /// Gui item to manage relation types.
    /// </summary>
    public partial class RelationTypeInput : TranslateableForm, IRelationTypeInputPresenter
    {
        #region Fields

        private const int width = 404;

        private RelationTypeModel presenter;
        private RelationType relationType = new RelationType();
        private Usage usage = Usage.Display;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="RelationTypeInput"/> class.
        /// </summary>
        public RelationTypeInput()
        {
            this.presenter = new RelationTypeModel(this);
            InitializeComponent();
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when user ask to delete the family link type.
        /// </summary>
        public event EventHandler Deleted;

        /// <summary>
        /// Occurs when user ask to insert the family link type.
        /// </summary>
        public event EventHandler Inserted;

        /// <summary>
        /// Occurs when user ask to update the family link type.
        /// </summary>
        public event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="InsuranceInput"/> is editable.
        /// </summary>
        /// <value><c>true</c> if editable; otherwise, <c>false</c>.</value>
        public bool Editable
        {
            set
            {
                foreach (var control in this.flp_Panel.Controls)
                {
                    if (control is ITitledControl) (control as ITitledControl).Enabled = value;
                }
            }
            get
            {
                return (from ITitledControl control in this.flp_Panel.Controls
                        select control
                    ).Count() > 0;
            }
        }

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public RelationType ItemToManage
        {
            get
            {
                return this.relationType;
            }
            set
            {
                this.relationType = value;
                this.BindData();
            }
        }

        /// <summary>
        /// Gets a value indicating whether this manager is in read only mode.
        /// </summary>
        /// <value><c>true</c> if read only; otherwise, <c>false</c>.</value>
        public bool ReadOnly
        {
            get
            {
                return (this.Usage == Usage.Display || this.Usage == Usage.Delete);
            }
        }

        /// <summary>
        /// Gets or sets the usage of this GUI item.
        /// Should be Insert, Update or Delete.
        /// </summary>
        /// <value>The usage.</value>
        public Usage Usage
        {
            get
            {
                return this.usage;
            }
            set
            {
                this.usage = value;
                switch (value)
                {
                    case Usage.Display:
                        this.btn_OK.Text = "OK".Translate();
                        this.Editable = false;
                        break;
                    case Usage.Insert:
                        this.ItemToManage = new RelationType();
                        this.btn_OK.Text = "Insert".Translate();
                        this.DialogResult = DialogResult.OK;
                        this.Editable = true;
                        break;
                    case Usage.Update:
                        this.btn_OK.Text = "Update".Translate();
                        this.DialogResult = DialogResult.OK;
                        this.Editable = true;
                        break;
                    case Usage.Delete:
                        this.btn_OK.Text = "Delete".Translate();
                        this.DialogResult = DialogResult.OK;
                        this.Editable = false;
                        break;
                    default:
                        Assert.FailsOnEnumeration(value);
                        break;
                }
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Clears the error.
        /// </summary>
        public void ClearError()
        {
            this.ep_Error.Clear();
            foreach (Control control in this.flp_Panel.Controls)
            {
                control.Width = width;
            }
        }

        /// <summary>
        /// Sets the error explaining the validation fail for
        /// the specified control.
        /// </summary>
        /// <param name="failure">Indicate where the validation failed.</param>
        /// <param name="message">The message.</param>
        public void SetError(RelationTypeFailedOn failure, string message)
        {
            Control control = new Control();
            switch (failure)
            {
                case RelationTypeFailedOn.EmptyName:
                    control = this.ttb_UpperRelation;
                    break;
                case RelationTypeFailedOn.UpsideDownEmptyName:
                    control = this.ttb_LowerSideRelation;
                    break;
                case RelationTypeFailedOn.Referenced:
                    PredifinedMsgBoxes.WarnOnReferencedItem();
                    break;
                default:
                    Assert.FailsOnEnumeration(failure);
                    break;
            }
            control.Width -= 20;
            this.ep_Error.SetError(control, message);
        }

        private void BindData()
        {
            foreach (Control control in this.flp_Panel.Controls)
            {
                control.Visible = false;
                control.DataBindings.Clear();
            }
            this.Size = new Size();

            #region Upper relation
            this.s_RelationUpperSide.Title = "Upper side of the relation".Translate();
            this.s_RelationUpperSide.Visible = true;

            this.ttb_UpperRelation.Value = this.ItemToManage.Title;
            this.ttb_UpperRelation.Visible = true;
            this.ttb_UpperRelation.DataBindings.Add("Text", this.ItemToManage, "Title");

            if (!string.IsNullOrEmpty(this.ItemToManage.Remark) || !this.ReadOnly)
            {
                this.tml_UpperRemark.Value = this.ItemToManage.Remark;
                this.tml_UpperRemark.Visible = true;
                this.tml_UpperRemark.DataBindings.Add("Text", this.ItemToManage, "Remark");
            }
            #endregion

            #region Lower relation
            this.s_RelationLowerSide.Title = "Lower side of the relation".Translate();
            this.s_RelationLowerSide.Visible = true;

            if (this.ItemToManage.UpsideDown == null) this.ItemToManage.UpsideDown = new RelationType();
            this.ttb_LowerSideRelation.Value = this.ItemToManage.Title;
            this.ttb_LowerSideRelation.Visible = true;
            this.ttb_LowerSideRelation.DataBindings.Add("Text", this.ItemToManage, "UpsideDown.Title");

            if (!string.IsNullOrEmpty(this.ItemToManage.Remark) || !this.ReadOnly)
            {
                this.tml_LowerRemark.Value = this.ItemToManage.UpsideDown.Remark;
                this.tml_LowerRemark.Visible = true;
                this.tml_LowerRemark.DataBindings.Add("Text", this.ItemToManage, "UpsideDown.Remark");
            }
            this.relationType.UpsideDown.UpsideDown = this.relationType;
            #endregion
        }

        private void btn_OK_Click(object sender, EventArgs e)
        {
            switch (this.Usage)
            {
                case Usage.Display:
                    this.Close();
                    break;
                case Usage.Insert:
                    if (this.presenter.ValidateData())
                    {
                        this.Inserted.Raise(this);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                case Usage.Update:
                    if (this.presenter.ValidateData())
                    {
                        this.Updated.Raise(this);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                case Usage.Delete:
                    if (this.presenter.ValidateData())
                    {
                        this.Deleted.Raise(this);
                        this.DialogResult = DialogResult.OK;
                        this.Close();
                    }
                    break;
                default:
                    Assert.FailsOnEnumeration(this.Usage);
                    break;
            }
        }

        #endregion Methods
    }
}