﻿namespace Phoenix.View.Management.Controls
{
    using System;
    using System.Collections.Generic;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Core.Management;
    using Phoenix.Presenter;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Sessions;
    using Phoenix.Translation;
    using Phoenix.View.Frame;

    /// <summary>
    /// Side menu to manage meetings
    /// </summary>
    public partial class CalendarSideMenu : UserControl, IMeetingSidePresenter
    {
        #region Fields

        private ILog Logger = LogManager.GetLogger(typeof(CalendarSideMenu));
        private MeetingSideModel model;
        private IList<WorkMeeting> todayMeetings;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="CalendarSideMenu"/> class.
        /// </summary>
        public CalendarSideMenu()
        {
            InitializeComponent();
            AppContext.MeetingsUpdated += (sender, e) => this.Refreshed.Raise(this);
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when user interface is refreshed.
        /// </summary>
        public event EventHandler Refreshed;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the meetings of today.
        /// </summary>
        /// <value>The meetings of today.</value>
        public IList<WorkMeeting> TodayMeetings
        {
            get
            {
                return this.todayMeetings;
            }
            set
            {
                this.todayMeetings = value;
                this.RefreshTodayPane();
            }
        }

        #endregion Properties

        #region Methods

        private void InsertEmptyDay()
        {
            this.lv_Today.Items.Add("There are no meeting today!".Translate(), "warning");
            this.lv_Today.Enabled = false;
        }

        private void InsertPatient(Patient patient)
        {
            var item = new ListViewItem(string.Format("{0} {1}"
                , patient.FirstName
                , patient.LastName.ToUpper()));

            item.ImageKey = (patient.Sex.ToLower() == "m")
                ? "male"
                : "female";
            item.Tag = patient;

            this.lv_Today.Items.Add(item);
            this.lv_Today.Enabled = true;
        }

        private void LoadSession(Session session)
        {
            var sideMenu = AppContext.Context["SideMenu"] as SideMenu;
            Assert.IsNotNull(sideMenu, "sideMenu", "The AppContext is not configured correctly: SideMenu is not set.");
            sideMenu.Add(session);
            sideMenu.Select(SideMenu.NavigationPane.Sessions);
        }

        private void lv_Today_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (this.lv_Today.SelectedItems.Count == 0) return;

            var patient = this.lv_Today.SelectedItems[0].Tag as Patient;
            if (patient == null)
            {
                Logger.Warn("The tag of the selected node doesn't contain a patient.");
                return;
            }
            var session = new Session(patient);
            this.LoadSession(session);
        }

        private void MeetingSideMenu_Load(object sender, EventArgs e)
        {
            if (!this.DesignMode) this.model = new MeetingSideModel(this);
            this.Refreshed.Raise(this);
        }

        private void mv_MonthView_SelectionChanged(object sender, EventArgs e)
        {
            var from = this.mv_MonthView.SelectionStart;
            AppContext.NotifyWeekSelected(from);
        }

        private void RefreshTodayPane()
        {
            if (this.todayMeetings == null) return;
            var patients = this.model.GetPatientsFor(DateTime.Today);

            this.lv_Today.Items.Clear();

            if (patients.Count == 0) this.InsertEmptyDay();
            else foreach (var each in patients) this.InsertPatient(each);
        }

        private void tsb_Refresh_Click(object sender, EventArgs e)
        {
            this.Refreshed.Raise(this);
        }

        #endregion Methods
    }
}