﻿namespace Phoenix.View.Management
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Drawing;
    using System.Linq;
    using System.Windows.Forms;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Core.Management;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Translation;
    using Phoenix.View.Helpers.Rtf;
    using Phoenix.View.Input;
    using Phoenix.View.Properties;

    /// <summary>
    /// Manage the medical cards of a patient
    /// </summary>
    public partial class MedicalCardManagerBox : TranslateableForm, IMedicalCardManagerPresenter
    {
        #region Fields

        private const string CLICK_ON_CARD_TO_CONTINUE = @"

                            Double click on a medical card to activate the editor";

        private IList<Macro> macros = new List<Macro>();
        private MedicalCard medicalCard;
        IList<MedicalCardType> medicalCardTypes = new List<MedicalCardType>();
        private MedicalCardModel model;
        private Patient patient;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="MedicalCardManagerBox"/> class.
        /// </summary>
        public MedicalCardManagerBox()
        {
            InitializeComponent();
            this.model = new MedicalCardModel(this);
            this.rtb_Card.Text = CLICK_ON_CARD_TO_CONTINUE;
            this.rtb_Card.Font = Settings.Default.Editor_Font;
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when the view is refreshed.
        /// </summary>
        public event EventHandler Refreshed;

        /// <summary>
        /// Occurs when a new type is created.
        /// </summary>
        public event EventHandler<EventArgs<MedicalCardType>> TypeCreated;

        /// <summary>
        /// Occurs when a medical card type is removed.
        /// </summary>
        public event EventHandler<EventArgs<MedicalCardType>> TypeRemoved;

        /// <summary>
        /// Occurs when a type is updated.
        /// </summary>
        public event EventHandler<EventArgs<MedicalCardType>> TypeUpdated;

        /// <summary>
        /// Occurs when the patient is updated.
        /// </summary>
        public event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets the description of the manager.
        /// </summary>
        /// <value>The description.</value>
        public string Description
        {
            get { return "Medical cards"; }
        }

        /// <summary>
        /// Gets the value of the icon tp show in the manager icon.
        /// </summary>
        /// <value>The icon.</value>
        public IconValue IconValue
        {
            get { return IconValue.MedicalCard; }
        }

        /// <summary>
        /// Gets or sets the macros.
        /// </summary>
        /// <value>The macros.</value>
        public IList<Macro> Macros
        {
            get { return this.macros; }
            set
            {
                this.macros = value;
                this.tssb_Macros.DropDownItems.Clear();
                foreach (var macro in value) this.AddMacro(macro);
            }
        }

        /// <summary>
        /// Gets or sets the medical card types.
        /// </summary>
        /// <value>The medical card types.</value>
        public IList<MedicalCardType> MedicalCardTypes
        {
            get { return this.medicalCardTypes; }
            set
            {
                this.medicalCardTypes = value;
                if (!this.IsDisposed) this.RefreshTreeView();
            }
        }

        /// <summary>
        /// Gets or sets the patient.
        /// </summary>
        /// <value>The patient.</value>
        public Patient Patient
        {
            get
            {
                return this.patient;
            }
            set
            {
                this.patient = value;
                this.RefreshTreeView();
                this.Text = string.Format("Medical card of {0} {1}"
                    , patient.FirstName
                    , this.patient.LastName.ToUpper());
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Called when the component has to be refreshed.
        /// </summary>
        public void RefreshData()
        {
            this.Refreshed.Raise(this);
        }

        /// <summary>
        /// Refreshes the data of the gui.
        /// </summary>
        public void RefreshGui()
        {
            this.RefreshTreeView();
        }

        /// <summary>
        /// Closes this manager.
        /// </summary>
        public void ShutDown()
        {
            this.OnClosing(new CancelEventArgs());
            this.Close();
        }

        private void Add(MedicalCard card)
        {
            var node = this.FindNode(card.Type);
            if (node == null) return;

            TreeNode subnode = new TreeNode(card.Title);
            subnode.Tag = card;
            subnode.ImageKey = "card";
            node.Nodes.Add(subnode);
        }

        private void Add(MedicalCardType medicalCardType)
        {
            if (Exists(medicalCardType)) return;

            var node = new TreeNode(medicalCardType.Title);
            node.Tag = medicalCardType;
            this.tv_Cards.Nodes[0].Nodes.Add(node);
        }

        private void AddMacro(Macro macro)
        {
            try
            {
                if (macro == null) return;
                var tsmi = new ToolStripMenuItem(macro.Title);
                tsmi.Tag = macro.Text;
                tsmi.Click += (sender, e) =>
                {
                    macro.Bind(this.patient);
                    this.rtb_Card.SelectedText = macro.BindedText;
                };
                this.tssb_Macros.DropDownItems.Add(tsmi);
            }
            catch (Exception ex)
            {
                this.Logger.Warn("An error occured while creating the macro", ex);
            }
        }

        private void AddNewCard(string title)
        {
            MedicalCard card = new MedicalCard();
            card.Title = title;
            card.Type = this.GetSelectedType();
            card.Remark = string.Empty;
            card.CreationDate
                = card.UpdateDate
                = DateTime.Now;
            this.Patient.Add(card);
            this.Refreshed.Raise(this);

            this.medicalCard = card;
            this.DisplayCard();
            this.rtb_Card.Font = Settings.Default.Editor_Font;
        }

        private void btn_Bold_Click(object sender, EventArgs e)
        {
            this.SetTextStyle(FontStyle.Bold, sender as ToolStripButton);
        }

        private void btn_Italic_Click(object sender, EventArgs e)
        {
            this.SetTextStyle(FontStyle.Italic, sender as ToolStripButton);
        }

        private void btn_Underline_Click(object sender, EventArgs e)
        {
            this.SetTextStyle(FontStyle.Underline, sender as ToolStripButton);
        }

        private void cms_Context_Opening(object sender, CancelEventArgs e)
        {
            var clickedNode = this.tv_Cards.SelectedNode;

            if (clickedNode == null) return;
            else if (clickedNode == null) e.Cancel = true;
            else
            {
                this.tsmi_AddCard.Visible = (clickedNode.Tag is MedicalCardType);
                this.tsmi_UpdateType.Visible = (clickedNode.Tag is MedicalCardType);

                this.tsmi_AddType.Visible = (clickedNode.Tag == null);

                this.tsmi_UpdateTitle.Visible = (clickedNode.Tag is MedicalCard);
                this.tsmi_DeleteCard.Visible = (clickedNode.Tag is MedicalCard);

                e.Cancel = false;
            }
        }

        private void DisplayCard()
        {
            this.lbl_Title.Text = this.medicalCard.Title;
            this.rtb_Card.Text = string.Empty;
            this.rtb_Card.Rtf = this.medicalCard.Remark;

            this.tssl_Created.Text = "Date created: {0}".Translate(this.medicalCard.CreationDate);
            this.tssl_Modified.Text = "Date modified: {0}".Translate(this.medicalCard.UpdateDate);
            this.tssl_Title.Text = "Title: {0}".Translate(this.medicalCard.Title);
            this.rtb_Card.Enabled
                = this.ts_CardEditor.Enabled
                = true;
            this.rtb_Card.SelectionStart = this.rtb_Card.Text.Length;
        }

        private bool Exists(MedicalCardType medicalCardType)
        {
            foreach (TreeNode node in this.tv_Cards.Nodes)
            {
                var type = node.Tag as MedicalCardType;
                if (type == null) continue;
                if (type.ID == medicalCardType.ID) return true;
            }
            return false;
        }

        private TreeNode FindNode(MedicalCardType medicalCardType)
        {
            return (from TreeNode node in this.tv_Cards.Nodes[0].Nodes
                    where node.Tag is MedicalCardType
                    && (node.Tag as MedicalCardType).ID == medicalCardType.ID
                    select node).FirstOrDefault();
        }

        private MedicalCardType GetSelectedType()
        {
            if (this.tv_Cards.SelectedNode.Tag is MedicalCardType)
            {
                return this.tv_Cards.SelectedNode.Tag as MedicalCardType;
            }
            else if (this.tv_Cards.SelectedNode.Parent.Tag is MedicalCardType)
            {
                return this.tv_Cards.SelectedNode.Parent.Tag as MedicalCardType;
            }
            else throw new InvalidOperationException("Impossible to find the type of the medical card to insert");
        }

        private void MedicalCardManagerBox_Bis_Load(object sender, EventArgs e)
        {
            this.Refreshed.Raise(this);
        }

        private void OnNewTypeCreated(MedicalCardType type)
        {
            this.TypeCreated.Raise(this, new EventArgs<MedicalCardType>(type));
        }

        private void OnTypeRemoved(MedicalCardType type)
        {
            this.TypeRemoved.Raise(this, new EventArgs<MedicalCardType>(type));
        }

        private void OnTypeUpdated(MedicalCardType type)
        {
            this.TypeUpdated.Raise(this, new EventArgs<MedicalCardType>(type));
        }

        private void OnUpdate()
        {
            if (this.medicalCard == null) return;
            this.medicalCard.Remark = this.rtb_Card.Rtf;
            this.medicalCard.UpdateDate = DateTime.Now;
            this.Updated.Raise(this);
        }

        private void RefreshTreeView()
        {
            this.tv_Cards.Nodes.Clear();
            this.tv_Cards.Nodes.Add("Medical cards".Translate());

            foreach (var type in this.MedicalCardTypes) this.Add(type);
            foreach (var card in this.Patient.MedicalCards) this.Add(card);
            this.tv_Cards.ExpandAll();
        }

        private void rtb_Card_MouseUp(object sender, MouseEventArgs e)
        {
            if (rtb_Card.SelectionFont == null) return;

            this.btn_Bold.Checked = (this.rtb_Card.SelectionFont.Bold);
            this.btn_Italic.Checked = (this.rtb_Card.SelectionFont.Italic);
            this.btn_Underline.Checked = (this.rtb_Card.SelectionFont.Underline);
        }

        private void SaveCard()
        {
            if (string.IsNullOrEmpty(this.rtb_Card.Text)) return;

            this.OnUpdate();
            this.tssl_Saved.Text = string.Format("Last automatic save at {0}", DateTime.Now.ToLongTimeString());
        }

        private void SetTextStyle(FontStyle fontStyle, ToolStripButton button)
        {
            Assert.IsNotNull(button, "button", "The button should be a 'ToolStripButton'");
            if (this.rtb_Card.SelectionFont == null) return;

            var style = this.rtb_Card.SelectionFont.Style;

            style = (!button.Checked)
                ? Rtf.RemoveStyle(style, fontStyle)
                : Rtf.AddStyle(style, fontStyle);

            this.rtb_Card.SelectionFont = new Font(this.rtb_Card.SelectionFont, style);
        }

        private void tsb_Bullet_Click(object sender, EventArgs e)
        {
            this.rtb_Card.SelectionBullet = !this.rtb_Card.SelectionBullet;
        }

        private void tsb_Color_Click(object sender, EventArgs e)
        {
            var dr = cd_Color.ShowDialog();
            if (dr != DialogResult.OK) return;

            this.rtb_Card.SelectionColor = cd_Color.Color;
        }

        private void tsb_Copy_Click(object sender, EventArgs e)
        {
            this.rtb_Card.Copy();
        }

        private void tsb_Cut_Click(object sender, EventArgs e)
        {
            this.rtb_Card.Cut();
        }

        private void tsb_Font_Click(object sender, EventArgs e)
        {
            var dr = this.fd_Font.ShowDialog();
            if (dr != DialogResult.OK) return;
            this.rtb_Card.Font = this.fd_Font.Font;
        }

        private void tsb_Highlight_Click(object sender, EventArgs e)
        {
            var dr = cd_Color.ShowDialog();
            if (dr != DialogResult.OK) return;

            this.rtb_Card.SelectionBackColor = cd_Color.Color;
        }

        private void tsb_ManageMacros_Click(object sender, EventArgs e)
        {
            var form = GuiManager.Get<IMacroInputPresenter>();
            GuiManager.ShowDialog(form);
            this.RefreshData();
        }

        private void tsb_Paste_Click(object sender, EventArgs e)
        {
            this.rtb_Card.Paste();
        }

        private void tsb_Redo_Click(object sender, EventArgs e)
        {
            this.rtb_Card.Redo();
        }

        private void tsb_Save_Click(object sender, EventArgs e)
        {
            this.Save();
        }

        private void Save()
        {
            var cur = new CursorSnapshot();
            cur.Set(Cursors.WaitCursor);
            this.OnUpdate();
            cur.Rollback();
        }

        private void tsb_Undo_Click(object sender, EventArgs e)
        {
            this.rtb_Card.Undo();
        }

        private void tsmi_AddCard_Click(object sender, EventArgs e)
        {
            TextBoxInput form = new TextBoxInput("Enter the title of the card");
            form.Value = DateTime.Now.ToString();

            var dr = form.ShowDialog();
            if (dr != DialogResult.OK) return;

            AddNewCard(form.Value);
        }

        private void tsmi_AddType_Click(object sender, EventArgs e)
        {
            var form = new MedicalCardTypeInput();
            var dr = form.ShowDialog();
            if (dr != DialogResult.OK) return;

            this.OnNewTypeCreated(form.ItemToManage);
        }

        private void tsmi_DeleteCard_Click(object sender, EventArgs e)
        {
            var dr = MessageBoxes.Question("Do you want to delete this card?");
            if (dr != DialogResult.Yes) return;

            var card = this.tv_Cards.SelectedNode.Tag as MedicalCard;
            this.Patient.Remove(card);
            this.Updated.Raise(this);

            this.rtb_Card.Enabled
                = this.tlp_Editor.Enabled
                = false;
            this.rtb_Card.Text = CLICK_ON_CARD_TO_CONTINUE.Translate();
        }

        private void tsmi_UpdateTitle_Click(object sender, EventArgs e)
        {
            var node = this.tv_Cards.SelectedNode;
            if (node == null) return;

            var card = node.Tag as MedicalCard;
            if (card == null) return;

            var form = new TextBoxInput("Title of the card".Translate());
            form.Value = card.Title;
            var dr = form.ShowDialog();

            if (dr != DialogResult.OK) return;

            card.Title = form.Value;
            this.Updated.Raise(this);
        }

        private void tsmi_UpdateType_Click(object sender, EventArgs e)
        {
            var form = new MedicalCardTypeInput(this.tv_Cards.SelectedNode.Tag as MedicalCardType);

            var dr = form.ShowDialog();
            if (dr != DialogResult.OK) return;

            this.OnTypeUpdated(form.ItemToManage);
        }

        private void tssb_CollapseAll_Click(object sender, EventArgs e)
        {
            this.tv_Cards.CollapseAll();
        }

        private void tssb_ExpandAll_Click(object sender, EventArgs e)
        {
            this.tv_Cards.ExpandAll();
        }

        private void tssb_Macros_ButtonClick(object sender, EventArgs e)
        {
            this.tssb_Macros.ShowDropDown();
        }

        private void tv_Cards_AfterSelect(object sender, TreeViewEventArgs e)
        {
            var type = e.Node.Tag as MedicalCardType;
            if (type != null) this.tb_TypeRemarks.Text = type.Remark;
        }

        private void tv_Cards_BeforeSelect(object sender, TreeViewCancelEventArgs e)
        {
            e.Node.SelectedImageKey = e.Node.ImageKey;
        }

        private void tv_Cards_Click(object sender, EventArgs e)
        {
            if (this.tv_Cards.SelectedNode == null) return;

            var type = this.tv_Cards.SelectedNode.Tag as MedicalCardType;
            if (type != null) this.tb_TypeRemarks.Text = type.Remark;
        }

        private void tv_Cards_DoubleClick(object sender, EventArgs e)
        {
            if (this.tv_Cards.SelectedNode == null) return;
            if (this.tv_Cards.SelectedNode.Tag as MedicalCard == null) return;

            var bufCard = this.tv_Cards.SelectedNode.Tag as MedicalCard;

            this.SaveCard();

            this.medicalCard = bufCard;
            this.DisplayCard();
        }

        private void tv_Cards_DragComplete(object sender, Phoenix.Helpers.Winforms.DragCompleteEventArgs e)
        {
            if (!(e.TargetNode.Tag is MedicalCardType)) return;
            if (!(e.SourceNode.Tag is MedicalCard)) return;

            var type = e.TargetNode.Tag as MedicalCardType;
            var card = e.SourceNode.Tag as MedicalCard;

            card.Type = type;
            this.Updated.Raise(this);
        }

        private void tv_Cards_DragCompleting(object sender, Phoenix.Helpers.Winforms.CancelDragEventArgs e)
        {
            if (!(e.TargetNode.Tag is MedicalCardType)) e.Cancel = true;
        }

        private void tv_Cards_DragStart(object sender, Phoenix.Helpers.Winforms.DragItemEventArgs e)
        {
            this.tv_Cards.AllowDrop = (e.Node.Tag is MedicalCard);
        }

        private void tv_Cards_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right) this.tv_Cards.SelectedNode = this.tv_Cards.GetNodeAt(e.X, e.Y);
        }

        #endregion Methods

        private void MedicalCardManagerBox_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.Save();
        }
    }
}