﻿namespace Phoenix.Presenter.Search
{
    using System;
    using System.Collections.Generic;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Search;
    using Phoenix.Presenter.Interfaces.Searcher;
    using Phoenix.Presenter.Search.Criterion;
    using Phoenix.Translation;
    using Phoenix.View.Helpers;
    using Phoenix.View.Properties;

    /// <summary>
    /// Provides search box to find Patients.
    /// </summary>
    public class PatientSearchBox : SearchBox, IPatientSearcher
    {
        #region Fields

        private const int DefaultSplitterDistance = 180;

        private SessionPatientSearcher presenter;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PatientSearchBox"/> class.
        /// </summary>
        public PatientSearchBox()
            : base()
        {
            this.Text = "Search patients";
            this.Logger = LogManager.GetLogger(typeof(PatientSearchBox));

            #region Columns
            base.AddListViewColumn("c_Name", "Name");
            base.AddListViewColumn("c_Surname", "Surname");
            #endregion

            #region Criterions
            this.presenter = new SessionPatientSearcher(this);
            TextCriterion surname = new TextCriterion();
            surname.Text = "Last name:".Translate();
            surname.CriterionName = "LastName";
            surname.IsActivated = true;
            surname.TabIndex = 0;
            base.Criterions.Add(surname);

            TextCriterion name = new TextCriterion();
            name.Text = "First name:".Translate();
            name.CriterionName = "FirstName";
            name.IsActivated = false;
            name.TabIndex = 1;
            base.Criterions.Add(name);

            DateTimeCriterion dob = new DateTimeCriterion(this.MinimumDate);
            dob.Text = "Date of birth:".Translate();
            dob.CriterionName = "DateOfBirth";
            dob.IsActivated = false;
            dob.TabIndex = 2;
            base.Criterions.Add(dob);

            DateTimeCriterion ins = new DateTimeCriterion(this.MinimumDate);
            ins.Text = "Inscription date:".Translate();
            ins.CriterionName = "Inscription";
            ins.IsActivated = false;
            ins.TabIndex = 3;
            base.Criterions.Add(ins);
            #endregion

            #region Configuration
            this.SetSplitterDistance();
            base.DefaultView = (SearchBox.SearchBoxView)Enum.Parse(typeof(SearchBox.SearchBoxView), Settings.Default.SearchBox_DefaultView);
            this.MinimumDate = Settings.Default.General_MinimumDate;
            #endregion
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when an item is selected on the GUI.
        /// </summary>
        public event EventHandler<EventArgs<Patient>> ItemSelected;

        /// <summary>
        /// Occurs when the user fire a search in the GUI
        /// </summary>
        public event EventHandler<EventArgs<Patient>> Searching;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets the criterion whose'll be used for the search.
        /// </summary>
        /// <value>The criterion.</value>
        public override object Criterion
        {
            get
            {
                Patient p = new Patient();
                CriterionControl ctrl;
                if ((ctrl = this.Criterions["FirstName"]) != null
                    && ctrl.IsActivated)
                {
                    p.FirstName = ctrl.Value as string;
                }

                if ((ctrl = this.Criterions["LastName"]) != null
                    && ctrl.IsActivated)
                {
                    p.LastName = ctrl.Value as string;
                }

                if ((ctrl = this.Criterions["DateOfBirth"]) != null)
                {
                    p.DateOfBirth = (ctrl.IsActivated)
                        ? (DateTime)ctrl.Value
                        : p.DateOfBirth = DateTime.MinValue;
                }

                if ((ctrl = this.Criterions["Inscription"]) != null)
                {
                    p.InscriptionDate = (ctrl.IsActivated)
                        ? (DateTime)ctrl.Value
                        : DateTime.MinValue;
                }

                if ((ctrl = this.Criterions["Mail"]) != null
                    && ctrl.IsActivated)
                {
                    p.Mail = ctrl.Value as string;
                }

                return p;
            }
        }

        /// <summary>
        /// Gets or sets the minimum date whereon a search can be done.
        /// </summary>
        /// <value>The min date.</value>
        private DateTime MinimumDate
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Clears previous error on validation failed.
        /// </summary>
        public void ClearError()
        {
            foreach (CriterionControl control in this.Criterions)
            {
                control.ClearError();
            }
        }

        /// <summary>
        /// Fills the specified items into the GUI.
        /// </summary>
        /// <param name="items">The items to load into the SearchBox.</param>
        public void Fill(IList<Patient> items)
        {
            try
            {
                this.ClearItemsFromListView();
                List<ListViewItem> lvitems = new List<ListViewItem>();
                foreach (Patient item in items)
                {
                    ListViewItem lvitem = new ListViewItem(item.LastName);
                    ListViewItem.ListViewSubItem firstnameItem
                        = new ListViewItem.ListViewSubItem(lvitem, item.FirstName);

                    //Set image for the message.
                    ItemType iType = (item.Sex == "F")
                                         ? ItemType.Woman
                                         : ItemType.Man;
                    this.SetImage(iType, lvitem);

                    lvitem.SubItems.Add(firstnameItem);
                    lvitem.Tag = item;
                    lvitems.Add(lvitem);
                }
                this.AddListViewItems(lvitems);
                this.AutoResizeListView();
                this.DeactivateProgressBar();
            }
            catch (ArgumentException ex)
            {
                base.Logger.Warn(ex);
                MessageBoxes.Warning("An error occured while filling the result: " + ex.Message);
            }
        }

        /// <summary>
        /// Sets the error explaining the validation failed because
        /// the inscription date is bigger than the date of birth.
        /// </summary>
        /// <param name="message">The message.</param>
        public void InvalidateInscription(string message)
        {
            this.Criterions["DateOfBirth"].SetError(message);
            this.Criterions["Inscription"].SetError(message);
        }

        /// <summary>
        /// Called when a Patient search is fired.
        /// </summary>
        protected override void OnSearch()
        {
            try
            {
                base.ActivateProgressBar();
                this.Searching.Raise(this, new EventArgs<Patient>(this.Criterion as Patient));
            }
            catch (Exception ex)
            {
                this.Logger.Error(ex);
                PredifinedMsgBoxes.WarnUnexpectedError();
            }
        }

        /// <summary>
        /// Called when a Patient selection is fired.
        /// </summary>
        protected override void OnSelection()
        {
            try
            {
                Patient patient = this.SelectedItem as Patient;
                this.ItemSelected.Raise(this, new EventArgs<Patient>(patient));
            }
            catch (Exception ex)
            {
                string msg = "An error occured during the selection of a patient: {0}";
                this.Logger.Error(string.Format(msg, ex.Message), ex);
                MessageBoxes.Error(msg.Translate(ex.Message));
            }
        }

        /// <summary>
        /// Provide custom checking before processing click in the search button.
        /// If this methods returns false, no further process for the click on search.
        /// </summary>
        /// <returns>
        /// 	<c>True</c> if the check succeed. Otherwise returns <c>False</c>
        /// </returns>
        protected override bool ValidateData()
        {
            //The inscription date should be after the date of birth!
            DateTimeCriterion inscriptionCriterion = base.Criterions["Inscription"] as DateTimeCriterion;
            DateTimeCriterion dobCriterion = base.Criterions["DateOfBirth"] as DateTimeCriterion;

            //Check validity of the criterions
            Assert.IsNotNull(inscriptionCriterion, "inscriptionCriterion");
            Assert.IsNotNull(dobCriterion, "dobCriterion");

            return this.presenter.ValidateData();
        }

        private void InitializeComponent()
        {
            this.SuspendLayout();
            //
            // PatientSearchBox
            //
            this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
            this.ClientSize = new System.Drawing.Size(496, 593);
            this.Name = "PatientSearchBox";
            this.Load += new System.EventHandler(this.PatientSearchBox_Load);
            this.ResumeLayout(false);
            this.PerformLayout();

            this.TranslateForm();
        }

        private void PatientSearchBox_Load(object sender, EventArgs e)
        {
        }

        #endregion Methods
    }
}